#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <tkhost.h>
#include <configf.h>
#include <tkiocomm.h>

/*
 * Planetbuster
 *
 * Demo add-on for planets toolkit
 */

/*
 * Planet code: con reports status
 *
 * Planet code: BPB at base builds planetbuster
 *
 * Ship code: LPB loads planetbuster (1/ship)
 *
 * Ship code: FPB fires planetbuster (from orbit)
 *
 * Ship/planet code: RPB reports inventory
 *
 * codes are reset to random after successful use
 */

int main(int argc,char *argv[])
{
  char strbuf[10];
  int costT,costD,costM;
  int t,p,pt;
  OUTMESS ms;
  SHIPDATA *shipbuffx;
  PLANETDATA *planbuffx;
  PLANETDATA *planbuff;
  BASEDATA *basebuff;
  PLANET_COORDS *pcbuffx;
  PLANET_NAME *pname;
  RACENM *rname;
  FILE *h,*log;
  UB1 pbi;

  set_game_dir(argv[1]);

  srandom(time(NULL));

  log=mkopen("pbuster.log",false,'o');
  if (!log) return 1;

  h=mkopen("pbuster.hst",false,'r');
  if (!h) {
    h=mkopen("pbuster.hst",false,'o');
    for (t=1;t<=1000;t++) {
      writeUB1(h,0);
    }
  }
  fclose(h);

  h=mkopen("pbuster.hst",false,'w');

  if (read_own_ini("pbuster.ini") > 0) {
    find_header("Planetbuster");
    costT=atoi((const char *)get_parameter("CostT"));
    costD=atoi((const char *)get_parameter("CostD"));
    costM=atoi((const char *)get_parameter("CostM"));
    close_ini();
  } else {
    costT=1000;
    costD=2000;
    costM=3000;
  }

  /* check all friendly codes, build PBs */

  planbuffx=read_all_planetdata();
  if (!planbuffx) {
    fprintf(log,"Failed to read planet data.\n");
    return 1;
  }
  shipbuffx=read_all_shipdata();
  if (!shipbuffx) {
    fprintf(log,"Failed to read ship data.\n");
    return 1;
  }

  for(t=1;t<=500;t++) {
    ms.iTo=planbuffx[t].wOwner;
    if (planbuffx[t].sCode.Lower()=="con") {
      fprintf(log,"Config request from %d\n",t);
      ms.sMessage="-(c0000) Planetbuster configuration:\r\r";
      ms.sMessage |= "Cost in Tritanium: ";
      sprintf(strbuf,"%d",costT);
      ms.sMessage |= strbuf;
      ms.sMessage |= "\rCost in Duranium: ";
      sprintf(strbuf,"%d",costD);
      ms.sMessage |= strbuf;
      ms.sMessage |= "\rCost in Molybdenum: ";
      sprintf(strbuf,"%d",costM);
      ms.sMessage |= strbuf;
      ms.sMessage |= "\r";
      write_outmess(0,&ms);
    }
    if (planbuffx[t].sCode=="BPB") {
      fprintf(log,"Construction request from %d\n",t);
      pname=read_planet_name(t);
      if ( (planbuffx[t].lTri >= costT) &&
           (planbuffx[t].lDur >= costD) &&
           (planbuffx[t].lMol >= costM) ) {
        basebuff=read_basedata(t);
        if (basebuff) {
          if (basebuff->wOwner == planbuffx[t].wOwner) {
            fseek(h,t-1,SEEK_SET);
            pbi=readUB1(h);
            if (pbi!=255) {
              fseek(h,t-1,SEEK_SET);
              writeUB1(h,++pbi);
              fprintf(log,"...successful\n");
              planbuffx[t].lTri -= costT;
              planbuffx[t].lDur -= costD;
              planbuffx[t].lMol -= costM;
              planbuffx[t].sCode = rcode();
              write_planetdata(t,&planbuffx[t]);
              ms.sMessage="-(a0001) Planetbuster construction:\r\r";
              ms.sMessage |="A planetbuster has been built at\r";
              ms.sMessage |=pname->sName;
              ms.sMessage |="\rNumber on planet is now ";
              sprintf(strbuf,"%d",pbi);
              ms.sMessage |=strbuf;
              ms.sMessage |="\r";
              write_outmess(0,&ms);
            } else { // already has planetbuster
              ms.sMessage="-(a0002) Planetbuster construction:\r\r";
              ms.sMessage |="Your planet ";
              ms.sMessage |=pname->sName;
              ms.sMessage |="\ralready has 255 planetbusters!\r";
              write_outmess(0,&ms);
              fprintf(log,"...maxed out\n");
            }
          } else { // no base
            ms.sMessage="-(a) Planetbuster construction:\r\r";
            ms.sMessage |="There is no starbase at\r";
            ms.sMessage |=pname->sName;
            ms.sMessage |=".";
            write_outmess(0,&ms);
            fprintf(log,"...no base\n");
          }
          delete basebuff;
        }
      } else { // insufficient minerals
        ms.sMessage="-(a) Planetbuster construction:\r\r";
        ms.sMessage |="There are not enough minerals at\r";
        ms.sMessage |=pname->sName;
        ms.sMessage |=".";
        write_outmess(0,&ms);
        fprintf(log,"...insufficient minerals\n");
      }
      delete pname;
    }
    if (planbuffx[t].sCode=="RPB") {
      pname=read_planet_name(t);
      fseek(h,t-1,SEEK_SET);
      pbi=readUB1(h);
      ms.iTo=planbuffx[t].wOwner;
      ms.sMessage="-(a)Planetbuster reporting:\r\r";
      ms.sMessage|=pname->sName;
      switch (pbi) {
        case 0:
          ms.sMessage|="has no planetbusters.\r";
          break;
        case 1:
          ms.sMessage|="has one planetbuster.\r";
          break;
        default:
          ms.sMessage|="has ";
          sprintf(strbuf,"%d",pbi);
          ms.sMessage|=strbuf;
          ms.sMessage|=" planetbusters.\r";
      }
      write_outmess(0,&ms);
      delete pname;
    }
  }

  delete planbuffx;

  /* check for PBs being loaded/fired */

  pcbuffx=read_all_planet_coords();

  shipbuffx=read_all_shipdata();
  if (!shipbuffx) {
    fprintf(log,"Failed to read ship data.\n");
    return 1;
  }

  for (t=1;t<501;t++) {
    ms.iTo=shipbuffx[t].wRace;
    if (shipbuffx[t].sCode=="RPB") {
      fseek(h,t+499,SEEK_SET);
      pbi=readUB1(h);
      ms.iTo=planbuffx[t].wOwner;
      ms.sMessage="-(a)Planetbuster reporting:\r\r";
      ms.sMessage|=shipbuffx[t].sName;
      switch (pbi) {
        case 0:
          ms.sMessage|="has no planetbuster loaded.\r";
          break;
        case 1:
          ms.sMessage|="has a planetbuster loaded.\r";
      }
      write_outmess(0,&ms);
    }
    if (shipbuffx[t].sCode=="LPB") {
      fprintf(log,"Load request from %d\n",t);
      fseek(h,t+499,SEEK_SET);
      pbi=readUB1(h);
      if (pbi==0) {
        pt=0;
        for (p=1;p<501;p++) {
          if ( (shipbuffx[t].xCoord == pcbuffx[p].x) &&
               (shipbuffx[t].yCoord == pcbuffx[p].y) ) {
            pt=p;
            p=501;
          }
        }
        if (pt>0) {
          pname=read_planet_name(t);
          basebuff=read_basedata(pt);
          if (basebuff->wID==pt) {
            if ((basebuff->wOwner == shipbuffx[t].wRace) ||
                (planbuffx[t].sCode == shipbuffx[t].sCode)) {
              fseek(h,pt-1,SEEK_SET);
              pbi=readUB1(h);
              if (pbi>0) {
                pbi--;
                fseek(h,pt-1,SEEK_SET);
                writeUB1(h,pbi);
                fseek(h,t+499,SEEK_SET);
                writeUB1(h,1);
                ms.sMessage="-(a) Planetbuster loading:\r\r";
                ms.sMessage |="Your ship ";
                ms.sMessage |=shipbuffx[t].sName;
                ms.sMessage |="\rhas loaded a planetbuster.\r";
                fprintf(log,"...successful\n");
              } else {
                //no PBs at base
                ms.sMessage="-(a) Planetbuster loading:\r\r";
                ms.sMessage |="There are no planetbusters at\r";
                ms.sMessage |=pname->sName;
                fprintf(log,"...no PBs available\n");
              }
            } else {
            //not yours, FC failure
            ms.sMessage="-(a) Planetbuster loading:\r\r";
            ms.sMessage |="Your ship ";
            ms.sMessage |=shipbuffx[t].sName;
            ms.sMessage |="\rfailed to load a planetbuster.\r";
            write_outmess(0,&ms);
            fprintf(log,"...authentication failure\n");
            }
          } else {
            //no base
            fprintf(log,"...no base\n");
            ms.sMessage="-(a) Planetbuster loading:\r\r";
            ms.sMessage |="Your ship ";
            ms.sMessage |=shipbuffx[t].sName;
            ms.sMessage |="\ris not at a starbase.\r";
            write_outmess(0,&ms);
          }
          delete pname;
        } else {
          //no planet
          ms.sMessage="-(a) Planetbuster loading:\r\r";
          ms.sMessage |="Your ship ";
          ms.sMessage |=shipbuffx[t].sName;
          ms.sMessage |="\ris not at a planet.\r";
          write_outmess(0,&ms);
          fprintf(log,"...no planet\n");
        }
      } else {
        ms.sMessage="-(a) Planetbuster loading:\r\r";
        ms.sMessage |="Your ship ";
        ms.sMessage |=shipbuffx[t].sName;
        ms.sMessage |="\ralready has a planetbuster\ron board.\r";
        write_outmess(0,&ms);
        fprintf(log,"...already on board\n");
      }
    }
    if (shipbuffx[t].sCode=="FPB") {
      fprintf(log,"Fire request from %d\n",t);
      shipbuffx[t].sCode=rcode();
      write_shipdata(t,&shipbuffx[t]);
      fseek(h,t-1,SEEK_SET);
      if (readUB1(h)==1) {
        fseek(h,t+499,SEEK_SET);
        writeUB1(h,0);
        pt=0;
        for (p=1;p<501;p++) {
          if ( (shipbuffx[t].xCoord == pcbuffx[p].x) &&
               (shipbuffx[t].yCoord == pcbuffx[p].y) ) {
            pt=p;
            p=501;
          }
        }
        if (pt>0) {
          fprintf(log,"...successfully at %d\n",pt);
          pname=read_planet_name(pt);
          rname=read_racenm(shipbuffx[t].wRace);
          planbuff=read_planetdata(pt);
          if (planbuff->wOwner != 0) {
            ms.iTo=planbuff->wOwner;
            ms.sMessage="-(a) EMERGENCY MESSAGE:\r\r";
            ms.sMessage |=pname->sName;
            ms.sMessage |="has been hit by\r";
            ms.sMessage |="a planetbuster, fired by\r";
            ms.sMessage |=rname->sFull;
            write_outmess(0,&ms);
          }
          for (p=1;p<12;p++) {
            if (p!=planbuff->wOwner && p!=shipbuffx[t].wRace) {
              ms.iTo=p;
              ms.sMessage="-(a) Planetbuster detonation:\r\r";
              ms.sMessage |=pname->sName;
              ms.sMessage |="has been hit by\r";
              ms.sMessage |="a planetbuster, fired by\r";
              ms.sMessage |=rname->sFull;
              write_outmess(0,&ms);
            }
          }
          ms.iTo=shipbuffx[t].wRace;
          planbuff->wOwner=0;
          planbuff->wMines=0;
          planbuff->wFactories=0;
          planbuff->wDefense=0;
          planbuff->lFuel+=planbuff->lFuel_P;
          planbuff->lTri+=planbuff->lTri_P;
          planbuff->lDur+=planbuff->lDur_P;
          planbuff->lMol+=planbuff->lMol_P;
          planbuff->lColonists=0;
          planbuff->lSupplies=0;
          planbuff->lMoney=0;
          planbuff->lFuel_P=0;
          planbuff->lTri_P=0;
          planbuff->lDur_P=0;
          planbuff->lMol_P=0;
          planbuff->wFuel_D=0;
          planbuff->wTri_D=0;
          planbuff->wDur_D=0;
          planbuff->wMol_D=0;
          planbuff->wColTax=0;
          planbuff->wNativeTax=0;
          planbuff->iColStatus=100;
          planbuff->iNativeStatus=100;
          planbuff->wGov=0;
          planbuff->lNatives=0;
          planbuff->wRace=0;
          planbuff->wPlanet=100;
          write_planetdata(pt,planbuff);
          ms.sMessage="-(a) Planetbuster activation:\r\r";
          ms.sMessage |="Your ship ";
          ms.sMessage |=shipbuffx[t].sName;
          ms.sMessage |="\rfired its planetbuster into:\r";
          ms.sMessage |=pname->sName;
          write_outmess(0,&ms);
          delete pname;
          delete planbuff;
          delete rname;
        } else { //missed!
          ms.sMessage="-(a) Planetbuster activation:\r\r";
          ms.sMessage |="Your ship ";
          ms.sMessage |=shipbuffx[t].sName;
          ms.sMessage |="\rfired its planetbuster in free space.";
          write_outmess(0,&ms);
          fprintf(log,"...into space.\n");
        }
      } else { // ship doesn't have planetbuster
        ms.sMessage="-(a) Planetbuster activation:\r\r";
        ms.sMessage |="Your ship ";
        ms.sMessage |=shipbuffx[t].sName;
        ms.sMessage |="\rdid not have a planetbuster installed.";
        write_outmess(0,&ms);
        fprintf(log,"...system not installed.\n");
      }
    }
  }

  fclose(h);

  if (pcbuffx) delete pcbuffx;

  delete shipbuffx;

  fclose(log);

  return 0;
}
